@@ -12,17 +12,21 @@
 #
 """Utilities and helper functions."""
 
+from __future__ import annotations
+
 import functools
 import inspect
 import logging as py_logging
 import os
 import time
-from typing import Any, Callable, Tuple, Type, Union   # noqa: H301
+from typing import Any, Callable, Optional, Type, Union   # noqa: H301
+import uuid as uuid_lib
 
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import strutils
 
+from os_brick import executor
 from os_brick.i18n import _
 from os_brick.privileged import nvmeof as priv_nvme
 from os_brick.privileged import rootwrap as priv_rootwrap
@@ -54,7 +58,7 @@ LOG = logging.getLogger(__name__)
 
 class retry_if_exit_code(tenacity.retry_if_exception):
     """Retry on ProcessExecutionError specific exit codes."""
-    def __init__(self, codes: Union[int, Tuple[int, ...]]):
+    def __init__(self, codes: Union[int, tuple[int, ...]]):
         self.codes = (codes,) if isinstance(codes, int) else codes
         super(retry_if_exit_code, self).__init__(self._check_exit_code)
 
@@ -66,9 +70,9 @@ class retry_if_exit_code(tenacity.retry_if_exception):
 
 def retry(retry_param: Union[None,
                              Type[Exception],
-                             Tuple[Type[Exception], ...],
+                             tuple[Type[Exception], ...],
                              int,
-                             Tuple[int, ...]],
+                             tuple[int, ...]],
           interval: float = 1,
           retries: int = 3,
           backoff_rate: float = 2,
@@ -217,7 +221,7 @@ def convert_str(text: Union[bytes, str]) -> str:
         return text
 
 
-def get_host_nqn():
+def get_host_nqn() -> Optional[str]:
     try:
         with open('/etc/nvme/hostnqn', 'r') as f:
             host_nqn = f.read().strip()
@@ -228,6 +232,23 @@ def get_host_nqn():
     return host_nqn
 
 
+def get_nvme_host_id(uuid: Optional[str]) -> Optional[str]:
+    """Get the nvme host id
+
+    If the hostid file doesn't exist create it either with the passed uuid or
+    a random one.
+    """
+    try:
+        with open('/etc/nvme/hostid', 'r') as f:
+            host_id = f.read().strip()
+    except IOError:
+        uuid = uuid or str(uuid_lib.uuid4())
+        host_id = priv_nvme.create_hostid(uuid)
+    except Exception:
+        host_id = None
+    return host_id
+
+
 def _symlink_name_from_device_path(device_path):
     """Generate symlink absolute path for encrypted devices.
 
@@ -328,7 +349,9 @@ def get_dev_path(connection_properties, device_info):
     return convert_str(res)
 
 
-def connect_volume_undo_prepare_result(f=None, unlink_after=False):
+def connect_volume_undo_prepare_result(
+        f: Optional[Callable] = None,
+        unlink_after: bool = False) -> Callable:
     """Decorator that returns the device path to how it was originally.
 
     WARNING: This decorator must be **the first** decorator of the method to
@@ -394,3 +417,49 @@ def connect_volume_undo_prepare_result(f=None, unlink_after=False):
     if f:
         return decorator(f)
     return decorator
+
+
+def get_device_size(executor: executor.Executor, device: str) -> Optional[int]:
+    """Get the size in bytes of a volume."""
+    (out, _err) = executor._execute('blockdev', '--getsize64',
+                                    device, run_as_root=True,
+                                    root_helper=executor._root_helper)
+    var = str(out.strip())
+    if var.isnumeric():
+        return int(var)
+    else:
+        return None
+
+
+def check_valid_device(executor: executor.Executor, path: str) -> bool:
+    cmd = ('dd', 'if=%(path)s' % {"path": path},
+           'of=/dev/null', 'count=1')
+    out, info = None, None
+    try:
+        out, info = executor._execute(*cmd, run_as_root=True,
+                                      root_helper=executor._root_helper)
+    except processutils.ProcessExecutionError as e:
+        LOG.error("Failed to access the device on the path "
+                  "%(path)s: %(error)s.",
+                  {"path": path, "error": e.stderr})
+        return False
+    # If the info is none, the path does not exist.
+    return info is not None
+
+
+class Anything(object):
+    """Object equal to everything."""
+    def __eq__(self, other):
+        return True
+
+    def __ne__(self, other):
+        return False
+
+    def __str__(self):
+        return '<Anything>'
+
+    __lt__ = __gt__ = __le__ = __ge__ = __ne__
+    __repr__ = __str__
+
+
+ANY = Anything()
